using Amazon.Lambda.Core;
using Amazon.Lambda.SQSEvents;
using System.Text.Json;

// Assembly attribute to enable the Lambda function's JSON input to be converted into a .NET class.
[assembly: LambdaSerializer(typeof(Amazon.Lambda.Serialization.SystemTextJson.DefaultLambdaJsonSerializer))]

namespace SQSConsumer;

public class Function
{
    DateTime _today;
    public Function()
    {
         _today = DateTime.Today;
    }
    public async Task FunctionHandler(SQSEvent sqsEvent, ILambdaContext context)
    {
        foreach(var message in sqsEvent.Records)
        {
            await ProcessMessageAsync(message, context);
        }
    }

    private async Task ProcessMessageAsync(SQSEvent.SQSMessage message, ILambdaContext context)
    {
        context.Logger.LogInformation($"Processed message {message.Body}");
        var person = JsonSerializer.Deserialize<Person>(message.Body);
        var age = _today.Year - person.DateOfBirth.Year;
        if (person.DateOfBirth.Date > _today.AddYears(-age)) age--;
        person.Age = age;
        context.Logger.LogInformation(person.ToString());        
        await Task.CompletedTask;
    }
}

public class Person
{
    public string FirstName { get; set; }
    public string LastName { get; set; }
    public DateTime DateOfBirth { get; set; }
    public int Age { get; set; }
    public override string ToString()
    {
        return $"{FirstName}  {LastName} was born on {DateOfBirth} and is {Age} years old";
    }
}